attribute vec4 I0;
attribute vec4 I1;
attribute vec4 WP0;
attribute vec4 WP1;
attribute vec4 WNU0;
attribute vec4 WNU1;
attribute vec4 WNV0;
attribute vec4 WNV1;
attribute vec2 TEX;

uniform vec3 P[230]; 

varying vec3 normal;
varying vec2 texCoord;
varying vec4 shadowTexCoord;
varying vec4 pos;

/// Trilinear lattice deformer using position and deformation gradient-based tangent space weights. 
/// Doug L. James, 1/12/07
void main() 
{
	/// EXTRACT VERTEX's 8 LATTICE POSITIONS FROM UNIFORM MEMORY:
	ivec4 i0 = ivec4(I0);
	ivec4 i1 = ivec4(I1);
	vec3 p0 = P[i0[0]];
	vec3 p1 = P[i0[1]];
	vec3 p2 = P[i0[2]];
	vec3 p3 = P[i0[3]];
	vec3 p4 = P[i1[0]];
	vec3 p5 = P[i1[1]];
	vec3 p6 = P[i1[2]];
	vec3 p7 = P[i1[3]];

	/// WEIGHTED POSITION:
	vec3 p      = WP0[0]*p0 + WP0[1]*p1 + WP0[2]*p2 + WP0[3]*p3 + WP1[0]*p4 + WP1[1]*p5 + WP1[2]*p6 + WP1[3]*p7;
	pos			= vec4(p[0], p[1], p[2], 1);
	vec4 pos2   = gl_ModelViewMatrix * pos;
	gl_Position = gl_ProjectionMatrix * pos2;

	/// WEIGHTED TANGENTS:
	vec3   u  = WNU0[0]*p0 + WNU0[1]*p1 + WNU0[2]*p2 + WNU0[3]*p3 + WNU1[0]*p4 + WNU1[1]*p5 + WNU1[2]*p6 + WNU1[3]*p7;
	vec3   v  = WNV0[0]*p0 + WNV0[1]*p1 + WNV0[2]*p2 + WNV0[3]*p3 + WNV1[0]*p4 + WNV1[1]*p5 + WNV1[2]*p6 + WNV1[3]*p7;
	vec3   n  = normalize(cross(u,v));

	normal    = n;

	// TEXTURE STUFF:
	texCoord = TEX;
	shadowTexCoord = gl_TextureMatrix[1] * pos;
}
